%% @file
% det_tall_t.m
%
% @author Lars Johannesen
%
% @section DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%

%%
% @brief Returns annotations with likely tall T-wave detections removed
% @param u3 U3 signal
% @param anns Annotations
% @param fs Sampling frequency
% @return [oanns] = Annotations without likely tall T-waves
%
function [oanns] = det_tall_t(u3, anns, fs)
%Returns annotations with likely tall T-wave detections removed
%
% u3 U3 signal
% anns Annotations
% fs Sampling frequency
%
%                                  DISCLAIMER                                  
% This software and documentation (the "Software") were developed at the Food  
% and Drug Administration (FDA) by employees of the Federal Government in the  

% United States Code, this work is not subject to copyright protection and is  
% in the public domain. Permission is hereby granted, free of charge, to any   
% person obtaining a copy of the Software, to deal in the Software without     
% restriction, including without limitation the rights to use, copy, modify,   
% merge, publish, distribute, sublicense, or sell copies of the Software or    
% derivatives, and to permit persons to whom the Software is furnished to do   
% so. FDA assumes no responsibility whatsoever for use by other parties of     
% the Software, its source code, documentation or compiled executables, and    
% makes no guarantees, expressed or implied, about its quality, reliability,   
%  or any other characteristic. Further, use of this code in no way implies    
% endorsement by the FDA or confers any advantage in regulatory decisions.     
% Although this software can be redistributed and/or modified freely, we       
% ask that any derivative works bear some notice that they are derived from    
% it, and any modified versions bear some notice that they have been modified. 
%                                                                              
% Written by: Lars Johannesen
	% Create tmp matrix
	tmp = zeros(length(anns), 6); 
	% 1: Location of current, 2: location of next, 3: U3 value of current, 4: U3 value of next
	% 5: 1/0 (1=Occurs within 250 ms, and is at least 20% smaller), 
	% 6: 5 but blanked if detections are not sufficiently consistent

	tmp(1:(end-1),1) = anns(1:(end-1));
	tmp(1:(end-1),2) = anns(2:(end));
	tmp(1:(end-1),3) = u3(tmp(1:(end-1),1));
	tmp(1:(end-1),4) = u3(tmp(1:(end-1),2));

	width = round(250 * fs/1000.);
	heightdiff = 1.2;
	tmp(:,5) = (tmp(:,2) - tmp(:,1)) < width & (tmp(:,3) > heightdiff * tmp(:,4));

	maxsd = 10;
	minoccur = 0.25;
	sd = std(tmp((tmp(:,5)==1),2) - tmp((tmp(:,5)==1),1));
	tmp(:,6) = tmp(:,5);
	if (sd > maxsd || sum(tmp(:,5)>minoccur*length(anns)))
		tmp(:,6) = 0;
	end

	oanns = setdiff(anns,tmp((tmp(:,6)==1),2));
end
